"""
apply_svd

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.outputs import _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class apply_svd(Operator):
    r"""Computes the coefficients (=U*Sigma) and VT components from SVD.


    Parameters
    ----------
    field_contaner_to_compress: FieldsContainer
        fields container to be compressed
    scalar_int: int
        number of vectors (r) to keep for the future reconstraction of the matrix A, ex. A[m,n]=coef[m,r]*VT[r,n], where coef=U*Sigma
    scalar_double: float
        threshold (precision) as a double, default value is 1e-7
    boolean: bool
        apply svd on the initial input data (true) or transposed (square matrix), default value is false

    Returns
    -------
    us_svd: FieldsContainer
        the output entity is a field container (time dependant); it contains the multiplication of two matrices, U and S, where A=U.S.Vt
    vt_svd: FieldsContainer
        the output entity is a field container (space dependant), containing the Vt, where A=U.S.Vt
    sigma: Field or FieldsContainer
        the output entity is a field (or a field container if input fc contains several labels, where field contains results per label), containing singular (S) values of the input data, where A=U.S.Vt

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.compression.apply_svd()

    >>> # Make input connections
    >>> my_field_contaner_to_compress = dpf.FieldsContainer()
    >>> op.inputs.field_contaner_to_compress.connect(my_field_contaner_to_compress)
    >>> my_scalar_int = int()
    >>> op.inputs.scalar_int.connect(my_scalar_int)
    >>> my_scalar_double = float()
    >>> op.inputs.scalar_double.connect(my_scalar_double)
    >>> my_boolean = bool()
    >>> op.inputs.boolean.connect(my_boolean)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.compression.apply_svd(
    ...     field_contaner_to_compress=my_field_contaner_to_compress,
    ...     scalar_int=my_scalar_int,
    ...     scalar_double=my_scalar_double,
    ...     boolean=my_boolean,
    ... )

    >>> # Get output data
    >>> result_us_svd = op.outputs.us_svd()
    >>> result_vt_svd = op.outputs.vt_svd()
    >>> result_sigma = op.outputs.sigma()
    """

    def __init__(
        self,
        field_contaner_to_compress=None,
        scalar_int=None,
        scalar_double=None,
        boolean=None,
        config=None,
        server=None,
    ):
        super().__init__(name="svd_operator", config=config, server=server)
        self._inputs = InputsApplySvd(self)
        self._outputs = OutputsApplySvd(self)
        if field_contaner_to_compress is not None:
            self.inputs.field_contaner_to_compress.connect(field_contaner_to_compress)
        if scalar_int is not None:
            self.inputs.scalar_int.connect(scalar_int)
        if scalar_double is not None:
            self.inputs.scalar_double.connect(scalar_double)
        if boolean is not None:
            self.inputs.boolean.connect(boolean)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Computes the coefficients (=U*Sigma) and VT components from SVD.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field_contaner_to_compress",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""fields container to be compressed""",
                ),
                1: PinSpecification(
                    name="scalar_int",
                    type_names=["int32"],
                    optional=False,
                    document=r"""number of vectors (r) to keep for the future reconstraction of the matrix A, ex. A[m,n]=coef[m,r]*VT[r,n], where coef=U*Sigma""",
                ),
                2: PinSpecification(
                    name="scalar_double",
                    type_names=["double"],
                    optional=False,
                    document=r"""threshold (precision) as a double, default value is 1e-7""",
                ),
                3: PinSpecification(
                    name="boolean",
                    type_names=["bool"],
                    optional=False,
                    document=r"""apply svd on the initial input data (true) or transposed (square matrix), default value is false""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="us_svd",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""the output entity is a field container (time dependant); it contains the multiplication of two matrices, U and S, where A=U.S.Vt""",
                ),
                1: PinSpecification(
                    name="vt_svd",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""the output entity is a field container (space dependant), containing the Vt, where A=U.S.Vt""",
                ),
                2: PinSpecification(
                    name="sigma",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document=r"""the output entity is a field (or a field container if input fc contains several labels, where field contains results per label), containing singular (S) values of the input data, where A=U.S.Vt""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="svd_operator", server=server)

    @property
    def inputs(self) -> InputsApplySvd:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsApplySvd.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsApplySvd:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsApplySvd.
        """
        return super().outputs


class InputsApplySvd(_Inputs):
    """Intermediate class used to connect user inputs to
    apply_svd operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.compression.apply_svd()
    >>> my_field_contaner_to_compress = dpf.FieldsContainer()
    >>> op.inputs.field_contaner_to_compress.connect(my_field_contaner_to_compress)
    >>> my_scalar_int = int()
    >>> op.inputs.scalar_int.connect(my_scalar_int)
    >>> my_scalar_double = float()
    >>> op.inputs.scalar_double.connect(my_scalar_double)
    >>> my_boolean = bool()
    >>> op.inputs.boolean.connect(my_boolean)
    """

    def __init__(self, op: Operator):
        super().__init__(apply_svd._spec().inputs, op)
        self._field_contaner_to_compress = Input(
            apply_svd._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._field_contaner_to_compress)
        self._scalar_int = Input(apply_svd._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scalar_int)
        self._scalar_double = Input(apply_svd._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._scalar_double)
        self._boolean = Input(apply_svd._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._boolean)

    @property
    def field_contaner_to_compress(self) -> Input:
        r"""Allows to connect field_contaner_to_compress input to the operator.

        fields container to be compressed

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.compression.apply_svd()
        >>> op.inputs.field_contaner_to_compress.connect(my_field_contaner_to_compress)
        >>> # or
        >>> op.inputs.field_contaner_to_compress(my_field_contaner_to_compress)
        """
        return self._field_contaner_to_compress

    @property
    def scalar_int(self) -> Input:
        r"""Allows to connect scalar_int input to the operator.

        number of vectors (r) to keep for the future reconstraction of the matrix A, ex. A[m,n]=coef[m,r]*VT[r,n], where coef=U*Sigma

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.compression.apply_svd()
        >>> op.inputs.scalar_int.connect(my_scalar_int)
        >>> # or
        >>> op.inputs.scalar_int(my_scalar_int)
        """
        return self._scalar_int

    @property
    def scalar_double(self) -> Input:
        r"""Allows to connect scalar_double input to the operator.

        threshold (precision) as a double, default value is 1e-7

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.compression.apply_svd()
        >>> op.inputs.scalar_double.connect(my_scalar_double)
        >>> # or
        >>> op.inputs.scalar_double(my_scalar_double)
        """
        return self._scalar_double

    @property
    def boolean(self) -> Input:
        r"""Allows to connect boolean input to the operator.

        apply svd on the initial input data (true) or transposed (square matrix), default value is false

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.compression.apply_svd()
        >>> op.inputs.boolean.connect(my_boolean)
        >>> # or
        >>> op.inputs.boolean(my_boolean)
        """
        return self._boolean


class OutputsApplySvd(_Outputs):
    """Intermediate class used to get outputs from
    apply_svd operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.compression.apply_svd()
    >>> # Connect inputs : op.inputs. ...
    >>> result_us_svd = op.outputs.us_svd()
    >>> result_vt_svd = op.outputs.vt_svd()
    >>> result_sigma = op.outputs.sigma()
    """

    def __init__(self, op: Operator):
        super().__init__(apply_svd._spec().outputs, op)
        self._us_svd = Output(apply_svd._spec().output_pin(0), 0, op)
        self._outputs.append(self._us_svd)
        self._vt_svd = Output(apply_svd._spec().output_pin(1), 1, op)
        self._outputs.append(self._vt_svd)
        self.sigma_as_field = Output(
            _modify_output_spec_with_one_type(apply_svd._spec().output_pin(2), "field"),
            2,
            op,
        )
        self._outputs.append(self.sigma_as_field)
        self.sigma_as_fields_container = Output(
            _modify_output_spec_with_one_type(
                apply_svd._spec().output_pin(2), "fields_container"
            ),
            2,
            op,
        )
        self._outputs.append(self.sigma_as_fields_container)

    @property
    def us_svd(self) -> Output:
        r"""Allows to get us_svd output of the operator

        the output entity is a field container (time dependant); it contains the multiplication of two matrices, U and S, where A=U.S.Vt

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.compression.apply_svd()
        >>> # Get the output from op.outputs. ...
        >>> result_us_svd = op.outputs.us_svd()
        """
        return self._us_svd

    @property
    def vt_svd(self) -> Output:
        r"""Allows to get vt_svd output of the operator

        the output entity is a field container (space dependant), containing the Vt, where A=U.S.Vt

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.compression.apply_svd()
        >>> # Get the output from op.outputs. ...
        >>> result_vt_svd = op.outputs.vt_svd()
        """
        return self._vt_svd
