"""
abc_weightings

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class abc_weightings(Operator):
    r"""Computes ABC-weightings for the amplitude spectrum in dB units.


    Parameters
    ----------
    fields_container: FieldsContainer
        data to be weighted in dB units.
    weighting_type: int
        if this pin is set to 0, the A-weighting is computed, 1 the B-weigting is computed and 2 the C-weightings is computed.
    shape_by_tf_scoping: bool
        if this pin is set to true, each field of the input fields container is defined by time freq scoping and not by ids. Default is false

    Returns
    -------
    weightings: FieldsContainer
        weighted data in dB units.

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.filter.abc_weightings()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_weighting_type = int()
    >>> op.inputs.weighting_type.connect(my_weighting_type)
    >>> my_shape_by_tf_scoping = bool()
    >>> op.inputs.shape_by_tf_scoping.connect(my_shape_by_tf_scoping)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.filter.abc_weightings(
    ...     fields_container=my_fields_container,
    ...     weighting_type=my_weighting_type,
    ...     shape_by_tf_scoping=my_shape_by_tf_scoping,
    ... )

    >>> # Get output data
    >>> result_weightings = op.outputs.weightings()
    """

    def __init__(
        self,
        fields_container=None,
        weighting_type=None,
        shape_by_tf_scoping=None,
        config=None,
        server=None,
    ):
        super().__init__(name="abc_weightings", config=config, server=server)
        self._inputs = InputsAbcWeightings(self)
        self._outputs = OutputsAbcWeightings(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if weighting_type is not None:
            self.inputs.weighting_type.connect(weighting_type)
        if shape_by_tf_scoping is not None:
            self.inputs.shape_by_tf_scoping.connect(shape_by_tf_scoping)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Computes ABC-weightings for the amplitude spectrum in dB units.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""data to be weighted in dB units.""",
                ),
                1: PinSpecification(
                    name="weighting_type",
                    type_names=["int32"],
                    optional=False,
                    document=r"""if this pin is set to 0, the A-weighting is computed, 1 the B-weigting is computed and 2 the C-weightings is computed.""",
                ),
                2: PinSpecification(
                    name="shape_by_tf_scoping",
                    type_names=["bool"],
                    optional=False,
                    document=r"""if this pin is set to true, each field of the input fields container is defined by time freq scoping and not by ids. Default is false""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="weightings",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""weighted data in dB units.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="abc_weightings", server=server)

    @property
    def inputs(self) -> InputsAbcWeightings:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsAbcWeightings.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsAbcWeightings:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsAbcWeightings.
        """
        return super().outputs


class InputsAbcWeightings(_Inputs):
    """Intermediate class used to connect user inputs to
    abc_weightings operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.filter.abc_weightings()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_weighting_type = int()
    >>> op.inputs.weighting_type.connect(my_weighting_type)
    >>> my_shape_by_tf_scoping = bool()
    >>> op.inputs.shape_by_tf_scoping.connect(my_shape_by_tf_scoping)
    """

    def __init__(self, op: Operator):
        super().__init__(abc_weightings._spec().inputs, op)
        self._fields_container = Input(abc_weightings._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields_container)
        self._weighting_type = Input(abc_weightings._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._weighting_type)
        self._shape_by_tf_scoping = Input(
            abc_weightings._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._shape_by_tf_scoping)

    @property
    def fields_container(self) -> Input:
        r"""Allows to connect fields_container input to the operator.

        data to be weighted in dB units.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.abc_weightings()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def weighting_type(self) -> Input:
        r"""Allows to connect weighting_type input to the operator.

        if this pin is set to 0, the A-weighting is computed, 1 the B-weigting is computed and 2 the C-weightings is computed.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.abc_weightings()
        >>> op.inputs.weighting_type.connect(my_weighting_type)
        >>> # or
        >>> op.inputs.weighting_type(my_weighting_type)
        """
        return self._weighting_type

    @property
    def shape_by_tf_scoping(self) -> Input:
        r"""Allows to connect shape_by_tf_scoping input to the operator.

        if this pin is set to true, each field of the input fields container is defined by time freq scoping and not by ids. Default is false

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.abc_weightings()
        >>> op.inputs.shape_by_tf_scoping.connect(my_shape_by_tf_scoping)
        >>> # or
        >>> op.inputs.shape_by_tf_scoping(my_shape_by_tf_scoping)
        """
        return self._shape_by_tf_scoping


class OutputsAbcWeightings(_Outputs):
    """Intermediate class used to get outputs from
    abc_weightings operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.filter.abc_weightings()
    >>> # Connect inputs : op.inputs. ...
    >>> result_weightings = op.outputs.weightings()
    """

    def __init__(self, op: Operator):
        super().__init__(abc_weightings._spec().outputs, op)
        self._weightings = Output(abc_weightings._spec().output_pin(0), 0, op)
        self._outputs.append(self._weightings)

    @property
    def weightings(self) -> Output:
        r"""Allows to get weightings output of the operator

        weighted data in dB units.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.abc_weightings()
        >>> # Get the output from op.outputs. ...
        >>> result_weightings = op.outputs.weightings()
        """
        return self._weightings
