"""
timefreq_band_pass

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class timefreq_band_pass(Operator):
    r"""The band pass filter returns all the values above (but not equal to) the
    minimum threshold value and below (but not equal to) the maximum
    threshold value in input.


    Parameters
    ----------
    time_freq_support: TimeFreqSupport
    min_threshold: float or Field
        A minimum threshold scalar or a field containing one value is expected.
    max_threshold: float or Field, optional
        A maximum threshold scalar or a field containing one value is expected.

    Returns
    -------
    time_freq_support: TimeFreqSupport
    scoping: Scoping

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.filter.timefreq_band_pass()

    >>> # Make input connections
    >>> my_time_freq_support = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support.connect(my_time_freq_support)
    >>> my_min_threshold = float()
    >>> op.inputs.min_threshold.connect(my_min_threshold)
    >>> my_max_threshold = float()
    >>> op.inputs.max_threshold.connect(my_max_threshold)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.filter.timefreq_band_pass(
    ...     time_freq_support=my_time_freq_support,
    ...     min_threshold=my_min_threshold,
    ...     max_threshold=my_max_threshold,
    ... )

    >>> # Get output data
    >>> result_time_freq_support = op.outputs.time_freq_support()
    >>> result_scoping = op.outputs.scoping()
    """

    def __init__(
        self,
        time_freq_support=None,
        min_threshold=None,
        max_threshold=None,
        config=None,
        server=None,
    ):
        super().__init__(name="core::timefreq::band_pass", config=config, server=server)
        self._inputs = InputsTimefreqBandPass(self)
        self._outputs = OutputsTimefreqBandPass(self)
        if time_freq_support is not None:
            self.inputs.time_freq_support.connect(time_freq_support)
        if min_threshold is not None:
            self.inputs.min_threshold.connect(min_threshold)
        if max_threshold is not None:
            self.inputs.max_threshold.connect(max_threshold)

    @staticmethod
    def _spec() -> Specification:
        description = r"""The band pass filter returns all the values above (but not equal to) the
minimum threshold value and below (but not equal to) the maximum
threshold value in input.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_freq_support",
                    type_names=["time_freq_support"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="min_threshold",
                    type_names=["double", "field"],
                    optional=False,
                    document=r"""A minimum threshold scalar or a field containing one value is expected.""",
                ),
                2: PinSpecification(
                    name="max_threshold",
                    type_names=["double", "field"],
                    optional=True,
                    document=r"""A maximum threshold scalar or a field containing one value is expected.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="time_freq_support",
                    type_names=["time_freq_support"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="core::timefreq::band_pass", server=server)

    @property
    def inputs(self) -> InputsTimefreqBandPass:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsTimefreqBandPass.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsTimefreqBandPass:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsTimefreqBandPass.
        """
        return super().outputs


class InputsTimefreqBandPass(_Inputs):
    """Intermediate class used to connect user inputs to
    timefreq_band_pass operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.filter.timefreq_band_pass()
    >>> my_time_freq_support = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support.connect(my_time_freq_support)
    >>> my_min_threshold = float()
    >>> op.inputs.min_threshold.connect(my_min_threshold)
    >>> my_max_threshold = float()
    >>> op.inputs.max_threshold.connect(my_max_threshold)
    """

    def __init__(self, op: Operator):
        super().__init__(timefreq_band_pass._spec().inputs, op)
        self._time_freq_support = Input(
            timefreq_band_pass._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._time_freq_support)
        self._min_threshold = Input(timefreq_band_pass._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._min_threshold)
        self._max_threshold = Input(timefreq_band_pass._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._max_threshold)

    @property
    def time_freq_support(self) -> Input:
        r"""Allows to connect time_freq_support input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.timefreq_band_pass()
        >>> op.inputs.time_freq_support.connect(my_time_freq_support)
        >>> # or
        >>> op.inputs.time_freq_support(my_time_freq_support)
        """
        return self._time_freq_support

    @property
    def min_threshold(self) -> Input:
        r"""Allows to connect min_threshold input to the operator.

        A minimum threshold scalar or a field containing one value is expected.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.timefreq_band_pass()
        >>> op.inputs.min_threshold.connect(my_min_threshold)
        >>> # or
        >>> op.inputs.min_threshold(my_min_threshold)
        """
        return self._min_threshold

    @property
    def max_threshold(self) -> Input:
        r"""Allows to connect max_threshold input to the operator.

        A maximum threshold scalar or a field containing one value is expected.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.timefreq_band_pass()
        >>> op.inputs.max_threshold.connect(my_max_threshold)
        >>> # or
        >>> op.inputs.max_threshold(my_max_threshold)
        """
        return self._max_threshold


class OutputsTimefreqBandPass(_Outputs):
    """Intermediate class used to get outputs from
    timefreq_band_pass operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.filter.timefreq_band_pass()
    >>> # Connect inputs : op.inputs. ...
    >>> result_time_freq_support = op.outputs.time_freq_support()
    >>> result_scoping = op.outputs.scoping()
    """

    def __init__(self, op: Operator):
        super().__init__(timefreq_band_pass._spec().outputs, op)
        self._time_freq_support = Output(
            timefreq_band_pass._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._time_freq_support)
        self._scoping = Output(timefreq_band_pass._spec().output_pin(1), 1, op)
        self._outputs.append(self._scoping)

    @property
    def time_freq_support(self) -> Output:
        r"""Allows to get time_freq_support output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.timefreq_band_pass()
        >>> # Get the output from op.outputs. ...
        >>> result_time_freq_support = op.outputs.time_freq_support()
        """
        return self._time_freq_support

    @property
    def scoping(self) -> Output:
        r"""Allows to get scoping output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.filter.timefreq_band_pass()
        >>> # Get the output from op.outputs. ...
        >>> result_scoping = op.outputs.scoping()
        """
        return self._scoping
