"""
elementary_data_selector_fc

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class elementary_data_selector_fc(Operator):
    r"""Creates a scalar fields container based on the selected elementary data
    for each field.


    Parameters
    ----------
    fields_container: FieldsContainer or Field
    elementary_data_index: int
        one or several elementary data indices that will be extracted from the initial field. For a field with a nature matrix, this extracts the line indices.
    elementary_data_index_2: int, optional
        For a field with nature matrix, this extracts the column indices.

    Returns
    -------
    fields_container: FieldsContainer

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.logic.elementary_data_selector_fc()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_elementary_data_index = int()
    >>> op.inputs.elementary_data_index.connect(my_elementary_data_index)
    >>> my_elementary_data_index_2 = int()
    >>> op.inputs.elementary_data_index_2.connect(my_elementary_data_index_2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.logic.elementary_data_selector_fc(
    ...     fields_container=my_fields_container,
    ...     elementary_data_index=my_elementary_data_index,
    ...     elementary_data_index_2=my_elementary_data_index_2,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        fields_container=None,
        elementary_data_index=None,
        elementary_data_index_2=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="elementary_data_selector_fc", config=config, server=server
        )
        self._inputs = InputsElementaryDataSelectorFc(self)
        self._outputs = OutputsElementaryDataSelectorFc(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if elementary_data_index is not None:
            self.inputs.elementary_data_index.connect(elementary_data_index)
        if elementary_data_index_2 is not None:
            self.inputs.elementary_data_index_2.connect(elementary_data_index_2)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Creates a scalar fields container based on the selected elementary data
for each field.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container", "field"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="elementary_data_index",
                    type_names=["int32", "vector<int32>"],
                    optional=False,
                    document=r"""one or several elementary data indices that will be extracted from the initial field. For a field with a nature matrix, this extracts the line indices.""",
                ),
                3: PinSpecification(
                    name="elementary_data_index_2",
                    type_names=["int32", "vector<int32>"],
                    optional=True,
                    document=r"""For a field with nature matrix, this extracts the column indices.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(
            name="elementary_data_selector_fc", server=server
        )

    @property
    def inputs(self) -> InputsElementaryDataSelectorFc:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsElementaryDataSelectorFc.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsElementaryDataSelectorFc:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsElementaryDataSelectorFc.
        """
        return super().outputs


class InputsElementaryDataSelectorFc(_Inputs):
    """Intermediate class used to connect user inputs to
    elementary_data_selector_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.elementary_data_selector_fc()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_elementary_data_index = int()
    >>> op.inputs.elementary_data_index.connect(my_elementary_data_index)
    >>> my_elementary_data_index_2 = int()
    >>> op.inputs.elementary_data_index_2.connect(my_elementary_data_index_2)
    """

    def __init__(self, op: Operator):
        super().__init__(elementary_data_selector_fc._spec().inputs, op)
        self._fields_container = Input(
            elementary_data_selector_fc._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._fields_container)
        self._elementary_data_index = Input(
            elementary_data_selector_fc._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._elementary_data_index)
        self._elementary_data_index_2 = Input(
            elementary_data_selector_fc._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._elementary_data_index_2)

    @property
    def fields_container(self) -> Input:
        r"""Allows to connect fields_container input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.elementary_data_selector_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def elementary_data_index(self) -> Input:
        r"""Allows to connect elementary_data_index input to the operator.

        one or several elementary data indices that will be extracted from the initial field. For a field with a nature matrix, this extracts the line indices.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.elementary_data_selector_fc()
        >>> op.inputs.elementary_data_index.connect(my_elementary_data_index)
        >>> # or
        >>> op.inputs.elementary_data_index(my_elementary_data_index)
        """
        return self._elementary_data_index

    @property
    def elementary_data_index_2(self) -> Input:
        r"""Allows to connect elementary_data_index_2 input to the operator.

        For a field with nature matrix, this extracts the column indices.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.elementary_data_selector_fc()
        >>> op.inputs.elementary_data_index_2.connect(my_elementary_data_index_2)
        >>> # or
        >>> op.inputs.elementary_data_index_2(my_elementary_data_index_2)
        """
        return self._elementary_data_index_2


class OutputsElementaryDataSelectorFc(_Outputs):
    """Intermediate class used to get outputs from
    elementary_data_selector_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.elementary_data_selector_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(elementary_data_selector_fc._spec().outputs, op)
        self._fields_container = Output(
            elementary_data_selector_fc._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self) -> Output:
        r"""Allows to get fields_container output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.elementary_data_selector_fc()
        >>> # Get the output from op.outputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """
        return self._fields_container
