"""
absolute_value_by_component

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class absolute_value_by_component(Operator):
    r"""Compute the absolute value of each data value of the input field, no
    norm performed.


    Parameters
    ----------
    field: Field or FieldsContainer or float
        field or fields container with only one field is expected

    Returns
    -------
    field: Field

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.absolute_value_by_component()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.absolute_value_by_component(
    ...     field=my_field,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(self, field=None, config=None, server=None):
        super().__init__(
            name="absolute_value_by_component", config=config, server=server
        )
        self._inputs = InputsAbsoluteValueByComponent(self)
        self._outputs = OutputsAbsoluteValueByComponent(self)
        if field is not None:
            self.inputs.field.connect(field)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Compute the absolute value of each data value of the input field, no
norm performed.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=[
                        "field",
                        "fields_container",
                        "double",
                        "vector<double>",
                    ],
                    optional=False,
                    document=r"""field or fields container with only one field is expected""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(
            name="absolute_value_by_component", server=server
        )

    @property
    def inputs(self) -> InputsAbsoluteValueByComponent:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsAbsoluteValueByComponent.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsAbsoluteValueByComponent:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsAbsoluteValueByComponent.
        """
        return super().outputs


class InputsAbsoluteValueByComponent(_Inputs):
    """Intermediate class used to connect user inputs to
    absolute_value_by_component operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.absolute_value_by_component()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    """

    def __init__(self, op: Operator):
        super().__init__(absolute_value_by_component._spec().inputs, op)
        self._field = Input(absolute_value_by_component._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)

    @property
    def field(self) -> Input:
        r"""Allows to connect field input to the operator.

        field or fields container with only one field is expected

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.absolute_value_by_component()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field


class OutputsAbsoluteValueByComponent(_Outputs):
    """Intermediate class used to get outputs from
    absolute_value_by_component operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.absolute_value_by_component()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(absolute_value_by_component._spec().outputs, op)
        self._field = Output(absolute_value_by_component._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self) -> Output:
        r"""Allows to get field output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.absolute_value_by_component()
        >>> # Get the output from op.outputs. ...
        >>> result_field = op.outputs.field()
        """
        return self._field
