"""
add_constant_fc

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class add_constant_fc(Operator):
    r"""Computes the sum of a field (in 0) and a scalar (in 1).


    Parameters
    ----------
    fields_container: FieldsContainer
        field or fields container with only one field is expected
    weights: float
        double or vector of double

    Returns
    -------
    fields_container: FieldsContainer

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.add_constant_fc()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_weights = float()
    >>> op.inputs.weights.connect(my_weights)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.add_constant_fc(
    ...     fields_container=my_fields_container,
    ...     weights=my_weights,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        fields_container=None,
        weights=None,
        config=None,
        server=None,
        ponderation=None,
    ):
        super().__init__(name="add_constant_fc", config=config, server=server)
        self._inputs = InputsAddConstantFc(self)
        self._outputs = OutputsAddConstantFc(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if weights is not None:
            self.inputs.weights.connect(weights)
        elif ponderation is not None:
            warn(
                DeprecationWarning(
                    f'Operator add_constant_fc: Input name "ponderation" is deprecated in favor of "weights".'
                )
            )
            self.inputs.weights.connect(ponderation)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Computes the sum of a field (in 0) and a scalar (in 1).
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""field or fields container with only one field is expected""",
                ),
                1: PinSpecification(
                    name="weights",
                    type_names=["double", "vector<double>"],
                    optional=False,
                    document=r"""double or vector of double""",
                    aliases=["ponderation"],
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="add_constant_fc", server=server)

    @property
    def inputs(self) -> InputsAddConstantFc:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsAddConstantFc.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsAddConstantFc:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsAddConstantFc.
        """
        return super().outputs


class InputsAddConstantFc(_Inputs):
    """Intermediate class used to connect user inputs to
    add_constant_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.add_constant_fc()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_weights = float()
    >>> op.inputs.weights.connect(my_weights)
    """

    def __init__(self, op: Operator):
        super().__init__(add_constant_fc._spec().inputs, op)
        self._fields_container = Input(add_constant_fc._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields_container)
        self._weights = Input(add_constant_fc._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._weights)

    @property
    def fields_container(self) -> Input:
        r"""Allows to connect fields_container input to the operator.

        field or fields container with only one field is expected

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.add_constant_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def weights(self) -> Input:
        r"""Allows to connect weights input to the operator.

        double or vector of double

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.add_constant_fc()
        >>> op.inputs.weights.connect(my_weights)
        >>> # or
        >>> op.inputs.weights(my_weights)
        """
        return self._weights

    def __getattr__(self, name):
        if name in ["ponderation"]:
            warn(
                DeprecationWarning(
                    f'Operator add_constant_fc: Input name "{name}" is deprecated in favor of "weights".'
                )
            )
            return self.weights
        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{name}'."
        )


class OutputsAddConstantFc(_Outputs):
    """Intermediate class used to get outputs from
    add_constant_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.add_constant_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(add_constant_fc._spec().outputs, op)
        self._fields_container = Output(add_constant_fc._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self) -> Output:
        r"""Allows to get fields_container output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.add_constant_fc()
        >>> # Get the output from op.outputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """
        return self._fields_container
