"""
norm_fc

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class norm_fc(Operator):
    r"""Computes the element-wise L2 norm of the field elementary data. This
    process is applied on each field of the input fields container.


    Parameters
    ----------
    fields_container: FieldsContainer
    scalar_int: int, optional
        Lp normalisation type, p = 1, 2, ...n - Default Lp=2

    Returns
    -------
    fields_container: FieldsContainer

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.norm_fc()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_scalar_int = int()
    >>> op.inputs.scalar_int.connect(my_scalar_int)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.norm_fc(
    ...     fields_container=my_fields_container,
    ...     scalar_int=my_scalar_int,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self, fields_container=None, scalar_int=None, config=None, server=None
    ):
        super().__init__(name="norm_fc", config=config, server=server)
        self._inputs = InputsNormFc(self)
        self._outputs = OutputsNormFc(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if scalar_int is not None:
            self.inputs.scalar_int.connect(scalar_int)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Computes the element-wise L2 norm of the field elementary data. This
process is applied on each field of the input fields container.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="scalar_int",
                    type_names=["int32"],
                    optional=True,
                    document=r"""Lp normalisation type, p = 1, 2, ...n - Default Lp=2""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="norm_fc", server=server)

    @property
    def inputs(self) -> InputsNormFc:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsNormFc.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsNormFc:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsNormFc.
        """
        return super().outputs


class InputsNormFc(_Inputs):
    """Intermediate class used to connect user inputs to
    norm_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.norm_fc()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_scalar_int = int()
    >>> op.inputs.scalar_int.connect(my_scalar_int)
    """

    def __init__(self, op: Operator):
        super().__init__(norm_fc._spec().inputs, op)
        self._fields_container = Input(norm_fc._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields_container)
        self._scalar_int = Input(norm_fc._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scalar_int)

    @property
    def fields_container(self) -> Input:
        r"""Allows to connect fields_container input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.norm_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def scalar_int(self) -> Input:
        r"""Allows to connect scalar_int input to the operator.

        Lp normalisation type, p = 1, 2, ...n - Default Lp=2

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.norm_fc()
        >>> op.inputs.scalar_int.connect(my_scalar_int)
        >>> # or
        >>> op.inputs.scalar_int(my_scalar_int)
        """
        return self._scalar_int


class OutputsNormFc(_Outputs):
    """Intermediate class used to get outputs from
    norm_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.norm_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(norm_fc._spec().outputs, op)
        self._fields_container = Output(norm_fc._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self) -> Output:
        r"""Allows to get fields_container output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.norm_fc()
        >>> # Get the output from op.outputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """
        return self._fields_container
