"""
qr_solve

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class qr_solve(Operator):
    r"""Computes the solution using QR factorization.


    Parameters
    ----------
    fields_container: FieldsContainer
        fields_container
    rhs: FieldsContainer
        fields_container

    Returns
    -------
    fields_container: FieldsContainer

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.qr_solve()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_rhs = dpf.FieldsContainer()
    >>> op.inputs.rhs.connect(my_rhs)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.qr_solve(
    ...     fields_container=my_fields_container,
    ...     rhs=my_rhs,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, fields_container=None, rhs=None, config=None, server=None):
        super().__init__(name="qrsolveOp", config=config, server=server)
        self._inputs = InputsQrSolve(self)
        self._outputs = OutputsQrSolve(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if rhs is not None:
            self.inputs.rhs.connect(rhs)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Computes the solution using QR factorization.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""fields_container""",
                ),
                1: PinSpecification(
                    name="rhs",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""fields_container""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="qrsolveOp", server=server)

    @property
    def inputs(self) -> InputsQrSolve:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsQrSolve.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsQrSolve:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsQrSolve.
        """
        return super().outputs


class InputsQrSolve(_Inputs):
    """Intermediate class used to connect user inputs to
    qr_solve operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.qr_solve()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_rhs = dpf.FieldsContainer()
    >>> op.inputs.rhs.connect(my_rhs)
    """

    def __init__(self, op: Operator):
        super().__init__(qr_solve._spec().inputs, op)
        self._fields_container = Input(qr_solve._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields_container)
        self._rhs = Input(qr_solve._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._rhs)

    @property
    def fields_container(self) -> Input:
        r"""Allows to connect fields_container input to the operator.

        fields_container

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.qr_solve()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def rhs(self) -> Input:
        r"""Allows to connect rhs input to the operator.

        fields_container

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.qr_solve()
        >>> op.inputs.rhs.connect(my_rhs)
        >>> # or
        >>> op.inputs.rhs(my_rhs)
        """
        return self._rhs


class OutputsQrSolve(_Outputs):
    """Intermediate class used to get outputs from
    qr_solve operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.qr_solve()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(qr_solve._spec().outputs, op)
        self._fields_container = Output(qr_solve._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self) -> Output:
        r"""Allows to get fields_container output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.qr_solve()
        >>> # Get the output from op.outputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """
        return self._fields_container
