"""
skin

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class skin(Operator):
    r"""Extracts a skin of the mesh in a new meshed region. The material ID of
    initial elements are propagated to their facets.


    Parameters
    ----------
    mesh: MeshedRegion
    mesh_scoping: Scoping, optional
        Nodal scoping to restrict the skin extraction to a set of nodes. If provided, a skin element is added to the skin mesh if all its nodes are in the scoping.
    duplicate_shell: bool, optional
        If input mesh contains shell elements, output mesh shell elements (boolean = 1) are duplicated, one per each orientation, or (boolean = 0) remain unchanged.
    add_beam: bool, optional
        If input mesh contains beam elements, output mesh beam elements (boolean = 1) are added or (boolean = 0) are ignored.

    Returns
    -------
    mesh: MeshedRegion
        Skin meshed region with facets and facets_to_ele property fields.
    nodes_mesh_scoping: Scoping
    map_new_elements_to_old:
    property_field_new_elements_to_old: PropertyField
        This property field provides, for each new face element ID (in the scoping), the corresponding 3D volume element index (in the data) it has been extracted from. The 3D volume element ID can be found with the element scoping of the input mesh.
    facet_indices: PropertyField
        This property field gives, for each new face element ID (in the scoping), the corresponding face index on the source 3D volume element. The 3D volume element can be extracted from the previous output.

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.skin()

    >>> # Make input connections
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_duplicate_shell = bool()
    >>> op.inputs.duplicate_shell.connect(my_duplicate_shell)
    >>> my_add_beam = bool()
    >>> op.inputs.add_beam.connect(my_add_beam)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.skin(
    ...     mesh=my_mesh,
    ...     mesh_scoping=my_mesh_scoping,
    ...     duplicate_shell=my_duplicate_shell,
    ...     add_beam=my_add_beam,
    ... )

    >>> # Get output data
    >>> result_mesh = op.outputs.mesh()
    >>> result_nodes_mesh_scoping = op.outputs.nodes_mesh_scoping()
    >>> result_map_new_elements_to_old = op.outputs.map_new_elements_to_old()
    >>> result_property_field_new_elements_to_old = op.outputs.property_field_new_elements_to_old()
    >>> result_facet_indices = op.outputs.facet_indices()
    """

    def __init__(
        self,
        mesh=None,
        mesh_scoping=None,
        duplicate_shell=None,
        add_beam=None,
        config=None,
        server=None,
    ):
        super().__init__(name="meshed_skin_sector", config=config, server=server)
        self._inputs = InputsSkin(self)
        self._outputs = OutputsSkin(self)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if duplicate_shell is not None:
            self.inputs.duplicate_shell.connect(duplicate_shell)
        if add_beam is not None:
            self.inputs.add_beam.connect(add_beam)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Extracts a skin of the mesh in a new meshed region. The material ID of
initial elements are propagated to their facets.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document=r"""Nodal scoping to restrict the skin extraction to a set of nodes. If provided, a skin element is added to the skin mesh if all its nodes are in the scoping.""",
                ),
                2: PinSpecification(
                    name="duplicate_shell",
                    type_names=["bool"],
                    optional=True,
                    document=r"""If input mesh contains shell elements, output mesh shell elements (boolean = 1) are duplicated, one per each orientation, or (boolean = 0) remain unchanged.""",
                ),
                3: PinSpecification(
                    name="add_beam",
                    type_names=["bool"],
                    optional=True,
                    document=r"""If input mesh contains beam elements, output mesh beam elements (boolean = 1) are added or (boolean = 0) are ignored.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document=r"""Skin meshed region with facets and facets_to_ele property fields.""",
                ),
                1: PinSpecification(
                    name="nodes_mesh_scoping",
                    type_names=["scoping"],
                    optional=False,
                    document=r"""""",
                ),
                2: PinSpecification(
                    name="map_new_elements_to_old",
                    type_names=["umap<int32,int32>"],
                    optional=False,
                    document=r"""""",
                ),
                3: PinSpecification(
                    name="property_field_new_elements_to_old",
                    type_names=["property_field"],
                    optional=False,
                    document=r"""This property field provides, for each new face element ID (in the scoping), the corresponding 3D volume element index (in the data) it has been extracted from. The 3D volume element ID can be found with the element scoping of the input mesh.""",
                ),
                4: PinSpecification(
                    name="facet_indices",
                    type_names=["property_field"],
                    optional=False,
                    document=r"""This property field gives, for each new face element ID (in the scoping), the corresponding face index on the source 3D volume element. The 3D volume element can be extracted from the previous output.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="meshed_skin_sector", server=server)

    @property
    def inputs(self) -> InputsSkin:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsSkin.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsSkin:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsSkin.
        """
        return super().outputs


class InputsSkin(_Inputs):
    """Intermediate class used to connect user inputs to
    skin operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.skin()
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_duplicate_shell = bool()
    >>> op.inputs.duplicate_shell.connect(my_duplicate_shell)
    >>> my_add_beam = bool()
    >>> op.inputs.add_beam.connect(my_add_beam)
    """

    def __init__(self, op: Operator):
        super().__init__(skin._spec().inputs, op)
        self._mesh = Input(skin._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._mesh)
        self._mesh_scoping = Input(skin._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh_scoping)
        self._duplicate_shell = Input(skin._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._duplicate_shell)
        self._add_beam = Input(skin._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._add_beam)

    @property
    def mesh(self) -> Input:
        r"""Allows to connect mesh input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def mesh_scoping(self) -> Input:
        r"""Allows to connect mesh_scoping input to the operator.

        Nodal scoping to restrict the skin extraction to a set of nodes. If provided, a skin element is added to the skin mesh if all its nodes are in the scoping.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def duplicate_shell(self) -> Input:
        r"""Allows to connect duplicate_shell input to the operator.

        If input mesh contains shell elements, output mesh shell elements (boolean = 1) are duplicated, one per each orientation, or (boolean = 0) remain unchanged.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> op.inputs.duplicate_shell.connect(my_duplicate_shell)
        >>> # or
        >>> op.inputs.duplicate_shell(my_duplicate_shell)
        """
        return self._duplicate_shell

    @property
    def add_beam(self) -> Input:
        r"""Allows to connect add_beam input to the operator.

        If input mesh contains beam elements, output mesh beam elements (boolean = 1) are added or (boolean = 0) are ignored.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> op.inputs.add_beam.connect(my_add_beam)
        >>> # or
        >>> op.inputs.add_beam(my_add_beam)
        """
        return self._add_beam


class OutputsSkin(_Outputs):
    """Intermediate class used to get outputs from
    skin operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.skin()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mesh = op.outputs.mesh()
    >>> result_nodes_mesh_scoping = op.outputs.nodes_mesh_scoping()
    >>> result_map_new_elements_to_old = op.outputs.map_new_elements_to_old()
    >>> result_property_field_new_elements_to_old = op.outputs.property_field_new_elements_to_old()
    >>> result_facet_indices = op.outputs.facet_indices()
    """

    def __init__(self, op: Operator):
        super().__init__(skin._spec().outputs, op)
        self._mesh = Output(skin._spec().output_pin(0), 0, op)
        self._outputs.append(self._mesh)
        self._nodes_mesh_scoping = Output(skin._spec().output_pin(1), 1, op)
        self._outputs.append(self._nodes_mesh_scoping)
        self._map_new_elements_to_old = Output(skin._spec().output_pin(2), 2, op)
        self._outputs.append(self._map_new_elements_to_old)
        self._property_field_new_elements_to_old = Output(
            skin._spec().output_pin(3), 3, op
        )
        self._outputs.append(self._property_field_new_elements_to_old)
        self._facet_indices = Output(skin._spec().output_pin(4), 4, op)
        self._outputs.append(self._facet_indices)

    @property
    def mesh(self) -> Output:
        r"""Allows to get mesh output of the operator

        Skin meshed region with facets and facets_to_ele property fields.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> # Get the output from op.outputs. ...
        >>> result_mesh = op.outputs.mesh()
        """
        return self._mesh

    @property
    def nodes_mesh_scoping(self) -> Output:
        r"""Allows to get nodes_mesh_scoping output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> # Get the output from op.outputs. ...
        >>> result_nodes_mesh_scoping = op.outputs.nodes_mesh_scoping()
        """
        return self._nodes_mesh_scoping

    @property
    def map_new_elements_to_old(self) -> Output:
        r"""Allows to get map_new_elements_to_old output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> # Get the output from op.outputs. ...
        >>> result_map_new_elements_to_old = op.outputs.map_new_elements_to_old()
        """
        return self._map_new_elements_to_old

    @property
    def property_field_new_elements_to_old(self) -> Output:
        r"""Allows to get property_field_new_elements_to_old output of the operator

        This property field provides, for each new face element ID (in the scoping), the corresponding 3D volume element index (in the data) it has been extracted from. The 3D volume element ID can be found with the element scoping of the input mesh.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> # Get the output from op.outputs. ...
        >>> result_property_field_new_elements_to_old = op.outputs.property_field_new_elements_to_old()
        """
        return self._property_field_new_elements_to_old

    @property
    def facet_indices(self) -> Output:
        r"""Allows to get facet_indices output of the operator

        This property field gives, for each new face element ID (in the scoping), the corresponding face index on the source 3D volume element. The 3D volume element can be extracted from the previous output.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.skin()
        >>> # Get the output from op.outputs. ...
        >>> result_facet_indices = op.outputs.facet_indices()
        """
        return self._facet_indices
