"""
integrate_over_time_freq

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class integrate_over_time_freq(Operator):
    r"""Integration of an input field over timefreq.


    Parameters
    ----------
    field: Field
    scoping: Scoping, optional
        Integrate the input field over a specific scoping.
    time_freq_support: TimeFreqSupport, optional
        Time Freq to integrate on, otherwise time freq support from the input field is taken.

    Returns
    -------
    field: Field

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.metadata.integrate_over_time_freq()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_time_freq_support = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support.connect(my_time_freq_support)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.metadata.integrate_over_time_freq(
    ...     field=my_field,
    ...     scoping=my_scoping,
    ...     time_freq_support=my_time_freq_support,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(
        self, field=None, scoping=None, time_freq_support=None, config=None, server=None
    ):
        super().__init__(name="timefreq::integrate", config=config, server=server)
        self._inputs = InputsIntegrateOverTimeFreq(self)
        self._outputs = OutputsIntegrateOverTimeFreq(self)
        if field is not None:
            self.inputs.field.connect(field)
        if scoping is not None:
            self.inputs.scoping.connect(scoping)
        if time_freq_support is not None:
            self.inputs.time_freq_support.connect(time_freq_support)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Integration of an input field over timefreq.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=True,
                    document=r"""Integrate the input field over a specific scoping.""",
                ),
                2: PinSpecification(
                    name="time_freq_support",
                    type_names=["time_freq_support"],
                    optional=True,
                    document=r"""Time Freq to integrate on, otherwise time freq support from the input field is taken.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="timefreq::integrate", server=server)

    @property
    def inputs(self) -> InputsIntegrateOverTimeFreq:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsIntegrateOverTimeFreq.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsIntegrateOverTimeFreq:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsIntegrateOverTimeFreq.
        """
        return super().outputs


class InputsIntegrateOverTimeFreq(_Inputs):
    """Intermediate class used to connect user inputs to
    integrate_over_time_freq operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.metadata.integrate_over_time_freq()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_time_freq_support = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support.connect(my_time_freq_support)
    """

    def __init__(self, op: Operator):
        super().__init__(integrate_over_time_freq._spec().inputs, op)
        self._field = Input(integrate_over_time_freq._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._scoping = Input(integrate_over_time_freq._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scoping)
        self._time_freq_support = Input(
            integrate_over_time_freq._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._time_freq_support)

    @property
    def field(self) -> Input:
        r"""Allows to connect field input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.integrate_over_time_freq()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def scoping(self) -> Input:
        r"""Allows to connect scoping input to the operator.

        Integrate the input field over a specific scoping.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.integrate_over_time_freq()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> # or
        >>> op.inputs.scoping(my_scoping)
        """
        return self._scoping

    @property
    def time_freq_support(self) -> Input:
        r"""Allows to connect time_freq_support input to the operator.

        Time Freq to integrate on, otherwise time freq support from the input field is taken.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.integrate_over_time_freq()
        >>> op.inputs.time_freq_support.connect(my_time_freq_support)
        >>> # or
        >>> op.inputs.time_freq_support(my_time_freq_support)
        """
        return self._time_freq_support


class OutputsIntegrateOverTimeFreq(_Outputs):
    """Intermediate class used to get outputs from
    integrate_over_time_freq operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.metadata.integrate_over_time_freq()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(integrate_over_time_freq._spec().outputs, op)
        self._field = Output(integrate_over_time_freq._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self) -> Output:
        r"""Allows to get field output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.metadata.integrate_over_time_freq()
        >>> # Get the output from op.outputs. ...
        >>> result_field = op.outputs.field()
        """
        return self._field
