"""
cms_subfile_info_provider

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class cms_subfile_info_provider(Operator):
    r"""Read required information from a subfile.


    Parameters
    ----------
    data_sources: DataSources
        Data_sources (must contain at least one subfile).
    cms_subfile_data: bool
        If this pin i set to true, data are return in a field.
    output_maxdof_on_masternodes: bool, optional
        If this pin is set to true, compute and add field with max degrees of freedom on master nodes

    Returns
    -------
    int32: int
        returns integer values in the order : unit system used, stiffness matrix present key, damping matrix present key, mass matrix present key, number of master nodes, number of virtual nodes
    field: PropertyField
        returns integer values in the order : number of load vectors (nvects), number of nodes (nnod), number of virtual nodes (nvnodes), number of modes (nvmodes)

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.cms_subfile_info_provider()

    >>> # Make input connections
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_cms_subfile_data = bool()
    >>> op.inputs.cms_subfile_data.connect(my_cms_subfile_data)
    >>> my_output_maxdof_on_masternodes = bool()
    >>> op.inputs.output_maxdof_on_masternodes.connect(my_output_maxdof_on_masternodes)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.cms_subfile_info_provider(
    ...     data_sources=my_data_sources,
    ...     cms_subfile_data=my_cms_subfile_data,
    ...     output_maxdof_on_masternodes=my_output_maxdof_on_masternodes,
    ... )

    >>> # Get output data
    >>> result_int32 = op.outputs.int32()
    >>> result_field = op.outputs.field()
    """

    def __init__(
        self,
        data_sources=None,
        cms_subfile_data=None,
        output_maxdof_on_masternodes=None,
        config=None,
        server=None,
    ):
        super().__init__(name="cms_subfile_info_provider", config=config, server=server)
        self._inputs = InputsCmsSubfileInfoProvider(self)
        self._outputs = OutputsCmsSubfileInfoProvider(self)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if cms_subfile_data is not None:
            self.inputs.cms_subfile_data.connect(cms_subfile_data)
        if output_maxdof_on_masternodes is not None:
            self.inputs.output_maxdof_on_masternodes.connect(
                output_maxdof_on_masternodes
            )

    @staticmethod
    def _spec() -> Specification:
        description = r"""Read required information from a subfile.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document=r"""Data_sources (must contain at least one subfile).""",
                ),
                200: PinSpecification(
                    name="cms_subfile_data",
                    type_names=["bool"],
                    optional=False,
                    document=r"""If this pin i set to true, data are return in a field.""",
                ),
                300: PinSpecification(
                    name="output_maxdof_on_masternodes",
                    type_names=["bool"],
                    optional=True,
                    document=r"""If this pin is set to true, compute and add field with max degrees of freedom on master nodes""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="int32",
                    type_names=["int32"],
                    optional=False,
                    document=r"""returns integer values in the order : unit system used, stiffness matrix present key, damping matrix present key, mass matrix present key, number of master nodes, number of virtual nodes""",
                ),
                1: PinSpecification(
                    name="field",
                    type_names=["property_field"],
                    optional=False,
                    document=r"""returns integer values in the order : number of load vectors (nvects), number of nodes (nnod), number of virtual nodes (nvnodes), number of modes (nvmodes)""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="cms_subfile_info_provider", server=server)

    @property
    def inputs(self) -> InputsCmsSubfileInfoProvider:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsCmsSubfileInfoProvider.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsCmsSubfileInfoProvider:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsCmsSubfileInfoProvider.
        """
        return super().outputs


class InputsCmsSubfileInfoProvider(_Inputs):
    """Intermediate class used to connect user inputs to
    cms_subfile_info_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.cms_subfile_info_provider()
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_cms_subfile_data = bool()
    >>> op.inputs.cms_subfile_data.connect(my_cms_subfile_data)
    >>> my_output_maxdof_on_masternodes = bool()
    >>> op.inputs.output_maxdof_on_masternodes.connect(my_output_maxdof_on_masternodes)
    """

    def __init__(self, op: Operator):
        super().__init__(cms_subfile_info_provider._spec().inputs, op)
        self._data_sources = Input(
            cms_subfile_info_provider._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)
        self._cms_subfile_data = Input(
            cms_subfile_info_provider._spec().input_pin(200), 200, op, -1
        )
        self._inputs.append(self._cms_subfile_data)
        self._output_maxdof_on_masternodes = Input(
            cms_subfile_info_provider._spec().input_pin(300), 300, op, -1
        )
        self._inputs.append(self._output_maxdof_on_masternodes)

    @property
    def data_sources(self) -> Input:
        r"""Allows to connect data_sources input to the operator.

        Data_sources (must contain at least one subfile).

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cms_subfile_info_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def cms_subfile_data(self) -> Input:
        r"""Allows to connect cms_subfile_data input to the operator.

        If this pin i set to true, data are return in a field.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cms_subfile_info_provider()
        >>> op.inputs.cms_subfile_data.connect(my_cms_subfile_data)
        >>> # or
        >>> op.inputs.cms_subfile_data(my_cms_subfile_data)
        """
        return self._cms_subfile_data

    @property
    def output_maxdof_on_masternodes(self) -> Input:
        r"""Allows to connect output_maxdof_on_masternodes input to the operator.

        If this pin is set to true, compute and add field with max degrees of freedom on master nodes

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cms_subfile_info_provider()
        >>> op.inputs.output_maxdof_on_masternodes.connect(my_output_maxdof_on_masternodes)
        >>> # or
        >>> op.inputs.output_maxdof_on_masternodes(my_output_maxdof_on_masternodes)
        """
        return self._output_maxdof_on_masternodes


class OutputsCmsSubfileInfoProvider(_Outputs):
    """Intermediate class used to get outputs from
    cms_subfile_info_provider operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.cms_subfile_info_provider()
    >>> # Connect inputs : op.inputs. ...
    >>> result_int32 = op.outputs.int32()
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(cms_subfile_info_provider._spec().outputs, op)
        self._int32 = Output(cms_subfile_info_provider._spec().output_pin(0), 0, op)
        self._outputs.append(self._int32)
        self._field = Output(cms_subfile_info_provider._spec().output_pin(1), 1, op)
        self._outputs.append(self._field)

    @property
    def int32(self) -> Output:
        r"""Allows to get int32 output of the operator

        returns integer values in the order : unit system used, stiffness matrix present key, damping matrix present key, mass matrix present key, number of master nodes, number of virtual nodes

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cms_subfile_info_provider()
        >>> # Get the output from op.outputs. ...
        >>> result_int32 = op.outputs.int32()
        """
        return self._int32

    @property
    def field(self) -> Output:
        r"""Allows to get field output of the operator

        returns integer values in the order : number of load vectors (nvects), number of nodes (nnod), number of virtual nodes (nvnodes), number of modes (nvmodes)

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.cms_subfile_info_provider()
        >>> # Get the output from op.outputs. ...
        >>> result_field = op.outputs.field()
        """
        return self._field
