"""
mapdl_section

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class mapdl_section(Operator):
    r"""Read the values of the section properties for a given section property
    field (property field that contains section information for each element
    of a mesh). The following keys can be used: Thickness, NumLayers. For
    layered elements, the following keys can be used: Thickness, MatID,
    Orientation, NumIntPoints.


    Parameters
    ----------
    properties_name: str
    section: PropertyField, optional
        Property field that contains a section id per element.(optional)
    streams_container: StreamsContainer
    data_sources: DataSources
    layer_property: bool, optional
        Property requested is layered. Default = False.
    layers_requested: optional
        Array with layers requested for the section. Default = AllLayers.

    Returns
    -------
    properties_value: FieldsContainer
    layers_per_section: PropertyField
        Only available if layer_property option is set to True.

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.mapdl_section()

    >>> # Make input connections
    >>> my_properties_name = str()
    >>> op.inputs.properties_name.connect(my_properties_name)
    >>> my_section = dpf.PropertyField()
    >>> op.inputs.section.connect(my_section)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_layer_property = bool()
    >>> op.inputs.layer_property.connect(my_layer_property)
    >>> my_layers_requested = dpf.()
    >>> op.inputs.layers_requested.connect(my_layers_requested)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.mapdl_section(
    ...     properties_name=my_properties_name,
    ...     section=my_section,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ...     layer_property=my_layer_property,
    ...     layers_requested=my_layers_requested,
    ... )

    >>> # Get output data
    >>> result_properties_value = op.outputs.properties_value()
    >>> result_layers_per_section = op.outputs.layers_per_section()
    """

    def __init__(
        self,
        properties_name=None,
        section=None,
        streams_container=None,
        data_sources=None,
        layer_property=None,
        layers_requested=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mapdl_section_properties", config=config, server=server)
        self._inputs = InputsMapdlSection(self)
        self._outputs = OutputsMapdlSection(self)
        if properties_name is not None:
            self.inputs.properties_name.connect(properties_name)
        if section is not None:
            self.inputs.section.connect(section)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if layer_property is not None:
            self.inputs.layer_property.connect(layer_property)
        if layers_requested is not None:
            self.inputs.layers_requested.connect(layers_requested)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Read the values of the section properties for a given section property
field (property field that contains section information for each element
of a mesh). The following keys can be used: Thickness, NumLayers. For
layered elements, the following keys can be used: Thickness, MatID,
Orientation, NumIntPoints.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="properties_name",
                    type_names=["string", "vector<string>"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="section",
                    type_names=["property_field"],
                    optional=True,
                    document=r"""Property field that contains a section id per element.(optional)""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=False,
                    document=r"""""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document=r"""""",
                ),
                5: PinSpecification(
                    name="layer_property",
                    type_names=["bool"],
                    optional=True,
                    document=r"""Property requested is layered. Default = False.""",
                ),
                6: PinSpecification(
                    name="layers_requested",
                    type_names=["vector<int32>"],
                    optional=True,
                    document=r"""Array with layers requested for the section. Default = AllLayers.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="properties_value",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="layers_per_section",
                    type_names=["property_field"],
                    optional=False,
                    document=r"""Only available if layer_property option is set to True.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="mapdl_section_properties", server=server)

    @property
    def inputs(self) -> InputsMapdlSection:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsMapdlSection.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsMapdlSection:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsMapdlSection.
        """
        return super().outputs


class InputsMapdlSection(_Inputs):
    """Intermediate class used to connect user inputs to
    mapdl_section operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.mapdl_section()
    >>> my_properties_name = str()
    >>> op.inputs.properties_name.connect(my_properties_name)
    >>> my_section = dpf.PropertyField()
    >>> op.inputs.section.connect(my_section)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_layer_property = bool()
    >>> op.inputs.layer_property.connect(my_layer_property)
    >>> my_layers_requested = dpf.()
    >>> op.inputs.layers_requested.connect(my_layers_requested)
    """

    def __init__(self, op: Operator):
        super().__init__(mapdl_section._spec().inputs, op)
        self._properties_name = Input(mapdl_section._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._properties_name)
        self._section = Input(mapdl_section._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._section)
        self._streams_container = Input(mapdl_section._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._streams_container)
        self._data_sources = Input(mapdl_section._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)
        self._layer_property = Input(mapdl_section._spec().input_pin(5), 5, op, -1)
        self._inputs.append(self._layer_property)
        self._layers_requested = Input(mapdl_section._spec().input_pin(6), 6, op, -1)
        self._inputs.append(self._layers_requested)

    @property
    def properties_name(self) -> Input:
        r"""Allows to connect properties_name input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.properties_name.connect(my_properties_name)
        >>> # or
        >>> op.inputs.properties_name(my_properties_name)
        """
        return self._properties_name

    @property
    def section(self) -> Input:
        r"""Allows to connect section input to the operator.

        Property field that contains a section id per element.(optional)

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.section.connect(my_section)
        >>> # or
        >>> op.inputs.section(my_section)
        """
        return self._section

    @property
    def streams_container(self) -> Input:
        r"""Allows to connect streams_container input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self) -> Input:
        r"""Allows to connect data_sources input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def layer_property(self) -> Input:
        r"""Allows to connect layer_property input to the operator.

        Property requested is layered. Default = False.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.layer_property.connect(my_layer_property)
        >>> # or
        >>> op.inputs.layer_property(my_layer_property)
        """
        return self._layer_property

    @property
    def layers_requested(self) -> Input:
        r"""Allows to connect layers_requested input to the operator.

        Array with layers requested for the section. Default = AllLayers.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> op.inputs.layers_requested.connect(my_layers_requested)
        >>> # or
        >>> op.inputs.layers_requested(my_layers_requested)
        """
        return self._layers_requested


class OutputsMapdlSection(_Outputs):
    """Intermediate class used to get outputs from
    mapdl_section operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.mapdl_section()
    >>> # Connect inputs : op.inputs. ...
    >>> result_properties_value = op.outputs.properties_value()
    >>> result_layers_per_section = op.outputs.layers_per_section()
    """

    def __init__(self, op: Operator):
        super().__init__(mapdl_section._spec().outputs, op)
        self._properties_value = Output(mapdl_section._spec().output_pin(0), 0, op)
        self._outputs.append(self._properties_value)
        self._layers_per_section = Output(mapdl_section._spec().output_pin(1), 1, op)
        self._outputs.append(self._layers_per_section)

    @property
    def properties_value(self) -> Output:
        r"""Allows to get properties_value output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> # Get the output from op.outputs. ...
        >>> result_properties_value = op.outputs.properties_value()
        """
        return self._properties_value

    @property
    def layers_per_section(self) -> Output:
        r"""Allows to get layers_per_section output of the operator

        Only available if layer_property option is set to True.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_section()
        >>> # Get the output from op.outputs. ...
        >>> result_layers_per_section = op.outputs.layers_per_section()
        """
        return self._layers_per_section
