"""
spectrum_data

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class spectrum_data(Operator):
    r"""Read spectral data from the result files contained in the streams or
    data sources.


    Parameters
    ----------
    streams: StreamsContainer, optional
        Result file container allowed to be kept open to cache data.
    data_sources: DataSources
        Result file path container, used if no streams are set.

    Returns
    -------
    participation_factors: FieldsContainer
        Fields container holding participation factors.
    mode_coefficients: FieldsContainer
        Fields container holding mode coefficients (PRS File).
    damping_ratios: FieldsContainer
        Fields container holding damping ratios (PRS File).
    global_damping: FieldsContainer
        Fields container holding for each spectrum: Global Damping Ratio, Damping Stiffness Coefficient & Damping Mass Coefficient (PRS File).
    missing_mass: FieldsContainer
        Fields container holding for each spectrum: Missing Mass Mode (0: None, 1: Active), Missing Mass Effect ZPA (PRS File).
    rigid_response: FieldsContainer
        Fields container holding for each spectrum: Rigid Response Mode (0: None, 1: Gupta, 2: Lindley), Freq Begin (Gupta) / ZPA (Lindley), Freq End (Gupta) (PRS File).

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.spectrum_data()

    >>> # Make input connections
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.spectrum_data(
    ...     streams=my_streams,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_participation_factors = op.outputs.participation_factors()
    >>> result_mode_coefficients = op.outputs.mode_coefficients()
    >>> result_damping_ratios = op.outputs.damping_ratios()
    >>> result_global_damping = op.outputs.global_damping()
    >>> result_missing_mass = op.outputs.missing_mass()
    >>> result_rigid_response = op.outputs.rigid_response()
    """

    def __init__(self, streams=None, data_sources=None, config=None, server=None):
        super().__init__(name="spectrum_data", config=config, server=server)
        self._inputs = InputsSpectrumData(self)
        self._outputs = OutputsSpectrumData(self)
        if streams is not None:
            self.inputs.streams.connect(streams)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Read spectral data from the result files contained in the streams or
data sources.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                3: PinSpecification(
                    name="streams",
                    type_names=["streams_container"],
                    optional=True,
                    document=r"""Result file container allowed to be kept open to cache data.""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document=r"""Result file path container, used if no streams are set.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="participation_factors",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""Fields container holding participation factors.""",
                ),
                1: PinSpecification(
                    name="mode_coefficients",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""Fields container holding mode coefficients (PRS File).""",
                ),
                2: PinSpecification(
                    name="damping_ratios",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""Fields container holding damping ratios (PRS File).""",
                ),
                3: PinSpecification(
                    name="global_damping",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""Fields container holding for each spectrum: Global Damping Ratio, Damping Stiffness Coefficient & Damping Mass Coefficient (PRS File).""",
                ),
                4: PinSpecification(
                    name="missing_mass",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""Fields container holding for each spectrum: Missing Mass Mode (0: None, 1: Active), Missing Mass Effect ZPA (PRS File).""",
                ),
                5: PinSpecification(
                    name="rigid_response",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""Fields container holding for each spectrum: Rigid Response Mode (0: None, 1: Gupta, 2: Lindley), Freq Begin (Gupta) / ZPA (Lindley), Freq End (Gupta) (PRS File).""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="spectrum_data", server=server)

    @property
    def inputs(self) -> InputsSpectrumData:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsSpectrumData.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsSpectrumData:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsSpectrumData.
        """
        return super().outputs


class InputsSpectrumData(_Inputs):
    """Intermediate class used to connect user inputs to
    spectrum_data operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.spectrum_data()
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(spectrum_data._spec().inputs, op)
        self._streams = Input(spectrum_data._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._streams)
        self._data_sources = Input(spectrum_data._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def streams(self) -> Input:
        r"""Allows to connect streams input to the operator.

        Result file container allowed to be kept open to cache data.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> op.inputs.streams.connect(my_streams)
        >>> # or
        >>> op.inputs.streams(my_streams)
        """
        return self._streams

    @property
    def data_sources(self) -> Input:
        r"""Allows to connect data_sources input to the operator.

        Result file path container, used if no streams are set.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsSpectrumData(_Outputs):
    """Intermediate class used to get outputs from
    spectrum_data operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.spectrum_data()
    >>> # Connect inputs : op.inputs. ...
    >>> result_participation_factors = op.outputs.participation_factors()
    >>> result_mode_coefficients = op.outputs.mode_coefficients()
    >>> result_damping_ratios = op.outputs.damping_ratios()
    >>> result_global_damping = op.outputs.global_damping()
    >>> result_missing_mass = op.outputs.missing_mass()
    >>> result_rigid_response = op.outputs.rigid_response()
    """

    def __init__(self, op: Operator):
        super().__init__(spectrum_data._spec().outputs, op)
        self._participation_factors = Output(spectrum_data._spec().output_pin(0), 0, op)
        self._outputs.append(self._participation_factors)
        self._mode_coefficients = Output(spectrum_data._spec().output_pin(1), 1, op)
        self._outputs.append(self._mode_coefficients)
        self._damping_ratios = Output(spectrum_data._spec().output_pin(2), 2, op)
        self._outputs.append(self._damping_ratios)
        self._global_damping = Output(spectrum_data._spec().output_pin(3), 3, op)
        self._outputs.append(self._global_damping)
        self._missing_mass = Output(spectrum_data._spec().output_pin(4), 4, op)
        self._outputs.append(self._missing_mass)
        self._rigid_response = Output(spectrum_data._spec().output_pin(5), 5, op)
        self._outputs.append(self._rigid_response)

    @property
    def participation_factors(self) -> Output:
        r"""Allows to get participation_factors output of the operator

        Fields container holding participation factors.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> # Get the output from op.outputs. ...
        >>> result_participation_factors = op.outputs.participation_factors()
        """
        return self._participation_factors

    @property
    def mode_coefficients(self) -> Output:
        r"""Allows to get mode_coefficients output of the operator

        Fields container holding mode coefficients (PRS File).

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> # Get the output from op.outputs. ...
        >>> result_mode_coefficients = op.outputs.mode_coefficients()
        """
        return self._mode_coefficients

    @property
    def damping_ratios(self) -> Output:
        r"""Allows to get damping_ratios output of the operator

        Fields container holding damping ratios (PRS File).

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> # Get the output from op.outputs. ...
        >>> result_damping_ratios = op.outputs.damping_ratios()
        """
        return self._damping_ratios

    @property
    def global_damping(self) -> Output:
        r"""Allows to get global_damping output of the operator

        Fields container holding for each spectrum: Global Damping Ratio, Damping Stiffness Coefficient & Damping Mass Coefficient (PRS File).

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> # Get the output from op.outputs. ...
        >>> result_global_damping = op.outputs.global_damping()
        """
        return self._global_damping

    @property
    def missing_mass(self) -> Output:
        r"""Allows to get missing_mass output of the operator

        Fields container holding for each spectrum: Missing Mass Mode (0: None, 1: Active), Missing Mass Effect ZPA (PRS File).

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> # Get the output from op.outputs. ...
        >>> result_missing_mass = op.outputs.missing_mass()
        """
        return self._missing_mass

    @property
    def rigid_response(self) -> Output:
        r"""Allows to get rigid_response output of the operator

        Fields container holding for each spectrum: Rigid Response Mode (0: None, 1: Gupta, 2: Lindley), Freq Begin (Gupta) / ZPA (Lindley), Freq End (Gupta) (PRS File).

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.spectrum_data()
        >>> # Get the output from op.outputs. ...
        >>> result_rigid_response = op.outputs.rigid_response()
        """
        return self._rigid_response
