"""
intersect

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class intersect(Operator):
    r"""Intersect 2 scopings and return the intersection and the difference
    between the intersection and the first scoping.


    Parameters
    ----------
    scopingA: Scoping
    scopingB: Scoping

    Returns
    -------
    intersection: Scoping
    scopingA_min_intersection: Scoping

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.intersect()

    >>> # Make input connections
    >>> my_scopingA = dpf.Scoping()
    >>> op.inputs.scopingA.connect(my_scopingA)
    >>> my_scopingB = dpf.Scoping()
    >>> op.inputs.scopingB.connect(my_scopingB)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.intersect(
    ...     scopingA=my_scopingA,
    ...     scopingB=my_scopingB,
    ... )

    >>> # Get output data
    >>> result_intersection = op.outputs.intersection()
    >>> result_scopingA_min_intersection = op.outputs.scopingA_min_intersection()
    """

    def __init__(self, scopingA=None, scopingB=None, config=None, server=None):
        super().__init__(name="scoping::intersect", config=config, server=server)
        self._inputs = InputsIntersect(self)
        self._outputs = OutputsIntersect(self)
        if scopingA is not None:
            self.inputs.scopingA.connect(scopingA)
        if scopingB is not None:
            self.inputs.scopingB.connect(scopingB)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Intersect 2 scopings and return the intersection and the difference
between the intersection and the first scoping.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="scopingA",
                    type_names=["scoping"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="scopingB",
                    type_names=["scoping"],
                    optional=False,
                    document=r"""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="intersection",
                    type_names=["scoping"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="scopingA_min_intersection",
                    type_names=["scoping"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="scoping::intersect", server=server)

    @property
    def inputs(self) -> InputsIntersect:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsIntersect.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsIntersect:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsIntersect.
        """
        return super().outputs


class InputsIntersect(_Inputs):
    """Intermediate class used to connect user inputs to
    intersect operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.intersect()
    >>> my_scopingA = dpf.Scoping()
    >>> op.inputs.scopingA.connect(my_scopingA)
    >>> my_scopingB = dpf.Scoping()
    >>> op.inputs.scopingB.connect(my_scopingB)
    """

    def __init__(self, op: Operator):
        super().__init__(intersect._spec().inputs, op)
        self._scopingA = Input(intersect._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._scopingA)
        self._scopingB = Input(intersect._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scopingB)

    @property
    def scopingA(self) -> Input:
        r"""Allows to connect scopingA input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> op.inputs.scopingA.connect(my_scopingA)
        >>> # or
        >>> op.inputs.scopingA(my_scopingA)
        """
        return self._scopingA

    @property
    def scopingB(self) -> Input:
        r"""Allows to connect scopingB input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> op.inputs.scopingB.connect(my_scopingB)
        >>> # or
        >>> op.inputs.scopingB(my_scopingB)
        """
        return self._scopingB


class OutputsIntersect(_Outputs):
    """Intermediate class used to get outputs from
    intersect operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.intersect()
    >>> # Connect inputs : op.inputs. ...
    >>> result_intersection = op.outputs.intersection()
    >>> result_scopingA_min_intersection = op.outputs.scopingA_min_intersection()
    """

    def __init__(self, op: Operator):
        super().__init__(intersect._spec().outputs, op)
        self._intersection = Output(intersect._spec().output_pin(0), 0, op)
        self._outputs.append(self._intersection)
        self._scopingA_min_intersection = Output(intersect._spec().output_pin(1), 1, op)
        self._outputs.append(self._scopingA_min_intersection)

    @property
    def intersection(self) -> Output:
        r"""Allows to get intersection output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> # Get the output from op.outputs. ...
        >>> result_intersection = op.outputs.intersection()
        """
        return self._intersection

    @property
    def scopingA_min_intersection(self) -> Output:
        r"""Allows to get scopingA_min_intersection output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.intersect()
        >>> # Get the output from op.outputs. ...
        >>> result_scopingA_min_intersection = op.outputs.scopingA_min_intersection()
        """
        return self._scopingA_min_intersection
