"""
on_named_selection

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class on_named_selection(Operator):
    r"""provides a scoping at a given location based on a given named selection


    Parameters
    ----------
    requested_location: str
    named_selection_name: str
        the string is expected to be in upper case
    int_inclusive: int, optional
        If element scoping is requested on a nodal named selection, if Inclusive == 1 then add all the elements adjacent to the nodes.If Inclusive == 0, only the elements which have all their nodes in the named selection are included
    streams_container: StreamsContainer, optional
    data_sources: DataSources

    Returns
    -------
    mesh_scoping: Scoping

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.on_named_selection()

    >>> # Make input connections
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_named_selection_name = str()
    >>> op.inputs.named_selection_name.connect(my_named_selection_name)
    >>> my_int_inclusive = int()
    >>> op.inputs.int_inclusive.connect(my_int_inclusive)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.on_named_selection(
    ...     requested_location=my_requested_location,
    ...     named_selection_name=my_named_selection_name,
    ...     int_inclusive=my_int_inclusive,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """

    def __init__(
        self,
        requested_location=None,
        named_selection_name=None,
        int_inclusive=None,
        streams_container=None,
        data_sources=None,
        config=None,
        server=None,
    ):
        super().__init__(name="scoping_provider_by_ns", config=config, server=server)
        self._inputs = InputsOnNamedSelection(self)
        self._outputs = OutputsOnNamedSelection(self)
        if requested_location is not None:
            self.inputs.requested_location.connect(requested_location)
        if named_selection_name is not None:
            self.inputs.named_selection_name.connect(named_selection_name)
        if int_inclusive is not None:
            self.inputs.int_inclusive.connect(int_inclusive)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec() -> Specification:
        description = r"""provides a scoping at a given location based on a given named selection
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="requested_location",
                    type_names=["string"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="named_selection_name",
                    type_names=["string"],
                    optional=False,
                    document=r"""the string is expected to be in upper case""",
                ),
                2: PinSpecification(
                    name="int_inclusive",
                    type_names=["int32"],
                    optional=True,
                    document=r"""If element scoping is requested on a nodal named selection, if Inclusive == 1 then add all the elements adjacent to the nodes.If Inclusive == 0, only the elements which have all their nodes in the named selection are included""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=True,
                    document=r"""""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document=r"""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="scoping_provider_by_ns", server=server)

    @property
    def inputs(self) -> InputsOnNamedSelection:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsOnNamedSelection.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsOnNamedSelection:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsOnNamedSelection.
        """
        return super().outputs


class InputsOnNamedSelection(_Inputs):
    """Intermediate class used to connect user inputs to
    on_named_selection operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.on_named_selection()
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_named_selection_name = str()
    >>> op.inputs.named_selection_name.connect(my_named_selection_name)
    >>> my_int_inclusive = int()
    >>> op.inputs.int_inclusive.connect(my_int_inclusive)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(on_named_selection._spec().inputs, op)
        self._requested_location = Input(
            on_named_selection._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._requested_location)
        self._named_selection_name = Input(
            on_named_selection._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._named_selection_name)
        self._int_inclusive = Input(on_named_selection._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._int_inclusive)
        self._streams_container = Input(
            on_named_selection._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._streams_container)
        self._data_sources = Input(on_named_selection._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def requested_location(self) -> Input:
        r"""Allows to connect requested_location input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> # or
        >>> op.inputs.requested_location(my_requested_location)
        """
        return self._requested_location

    @property
    def named_selection_name(self) -> Input:
        r"""Allows to connect named_selection_name input to the operator.

        the string is expected to be in upper case

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.named_selection_name.connect(my_named_selection_name)
        >>> # or
        >>> op.inputs.named_selection_name(my_named_selection_name)
        """
        return self._named_selection_name

    @property
    def int_inclusive(self) -> Input:
        r"""Allows to connect int_inclusive input to the operator.

        If element scoping is requested on a nodal named selection, if Inclusive == 1 then add all the elements adjacent to the nodes.If Inclusive == 0, only the elements which have all their nodes in the named selection are included

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.int_inclusive.connect(my_int_inclusive)
        >>> # or
        >>> op.inputs.int_inclusive(my_int_inclusive)
        """
        return self._int_inclusive

    @property
    def streams_container(self) -> Input:
        r"""Allows to connect streams_container input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self) -> Input:
        r"""Allows to connect data_sources input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_named_selection()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsOnNamedSelection(_Outputs):
    """Intermediate class used to get outputs from
    on_named_selection operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.on_named_selection()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    """

    def __init__(self, op: Operator):
        super().__init__(on_named_selection._spec().outputs, op)
        self._mesh_scoping = Output(on_named_selection._spec().output_pin(0), 0, op)
        self._outputs.append(self._mesh_scoping)

    @property
    def mesh_scoping(self) -> Output:
        r"""Allows to get mesh_scoping output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.on_named_selection()
        >>> # Get the output from op.outputs. ...
        >>> result_mesh_scoping = op.outputs.mesh_scoping()
        """
        return self._mesh_scoping
