"""
vtk_to_fields

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class vtk_to_fields(Operator):
    r"""Write a field based on a vtk file.


    Parameters
    ----------
    field_name: str, optional
        name of the field in the vtk file
    streams: StreamsContainer, optional
    data_sources: DataSources

    Returns
    -------
    fields_container: FieldsContainer
        fields_container

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.vtk_to_fields()

    >>> # Make input connections
    >>> my_field_name = str()
    >>> op.inputs.field_name.connect(my_field_name)
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.vtk_to_fields(
    ...     field_name=my_field_name,
    ...     streams=my_streams,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self, field_name=None, streams=None, data_sources=None, config=None, server=None
    ):
        super().__init__(name="vtk::vtk::FieldProvider", config=config, server=server)
        self._inputs = InputsVtkToFields(self)
        self._outputs = OutputsVtkToFields(self)
        if field_name is not None:
            self.inputs.field_name.connect(field_name)
        if streams is not None:
            self.inputs.streams.connect(streams)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Write a field based on a vtk file.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field_name",
                    type_names=["string"],
                    optional=True,
                    document=r"""name of the field in the vtk file""",
                ),
                3: PinSpecification(
                    name="streams",
                    type_names=["streams_container"],
                    optional=True,
                    document=r"""""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document=r"""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""fields_container""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="vtk::vtk::FieldProvider", server=server)

    @property
    def inputs(self) -> InputsVtkToFields:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsVtkToFields.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsVtkToFields:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsVtkToFields.
        """
        return super().outputs


class InputsVtkToFields(_Inputs):
    """Intermediate class used to connect user inputs to
    vtk_to_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.vtk_to_fields()
    >>> my_field_name = str()
    >>> op.inputs.field_name.connect(my_field_name)
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(vtk_to_fields._spec().inputs, op)
        self._field_name = Input(vtk_to_fields._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field_name)
        self._streams = Input(vtk_to_fields._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._streams)
        self._data_sources = Input(vtk_to_fields._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)

    @property
    def field_name(self) -> Input:
        r"""Allows to connect field_name input to the operator.

        name of the field in the vtk file

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_to_fields()
        >>> op.inputs.field_name.connect(my_field_name)
        >>> # or
        >>> op.inputs.field_name(my_field_name)
        """
        return self._field_name

    @property
    def streams(self) -> Input:
        r"""Allows to connect streams input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_to_fields()
        >>> op.inputs.streams.connect(my_streams)
        >>> # or
        >>> op.inputs.streams(my_streams)
        """
        return self._streams

    @property
    def data_sources(self) -> Input:
        r"""Allows to connect data_sources input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_to_fields()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsVtkToFields(_Outputs):
    """Intermediate class used to get outputs from
    vtk_to_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.vtk_to_fields()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(vtk_to_fields._spec().outputs, op)
        self._fields_container = Output(vtk_to_fields._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self) -> Output:
        r"""Allows to get fields_container output of the operator

        fields_container

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.vtk_to_fields()
        >>> # Get the output from op.outputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """
        return self._fields_container
