"""
forward_fields_container

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class forward_fields_container(Operator):
    r"""Returns the input field or fields container.


    Parameters
    ----------
    fields: FieldsContainer or Field

    Returns
    -------
    fields_container: FieldsContainer

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.forward_fields_container()

    >>> # Make input connections
    >>> my_fields = dpf.FieldsContainer()
    >>> op.inputs.fields.connect(my_fields)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.forward_fields_container(
    ...     fields=my_fields,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, fields=None, config=None, server=None):
        super().__init__(name="forward_fc", config=config, server=server)
        self._inputs = InputsForwardFieldsContainer(self)
        self._outputs = OutputsForwardFieldsContainer(self)
        if fields is not None:
            self.inputs.fields.connect(fields)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Returns the input field or fields container.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields",
                    type_names=["fields_container", "field"],
                    optional=False,
                    document=r"""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="forward_fc", server=server)

    @property
    def inputs(self) -> InputsForwardFieldsContainer:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsForwardFieldsContainer.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsForwardFieldsContainer:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsForwardFieldsContainer.
        """
        return super().outputs


class InputsForwardFieldsContainer(_Inputs):
    """Intermediate class used to connect user inputs to
    forward_fields_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.forward_fields_container()
    >>> my_fields = dpf.FieldsContainer()
    >>> op.inputs.fields.connect(my_fields)
    """

    def __init__(self, op: Operator):
        super().__init__(forward_fields_container._spec().inputs, op)
        self._fields = Input(forward_fields_container._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields)

    @property
    def fields(self) -> Input:
        r"""Allows to connect fields input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.forward_fields_container()
        >>> op.inputs.fields.connect(my_fields)
        >>> # or
        >>> op.inputs.fields(my_fields)
        """
        return self._fields


class OutputsForwardFieldsContainer(_Outputs):
    """Intermediate class used to get outputs from
    forward_fields_container operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.forward_fields_container()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(forward_fields_container._spec().outputs, op)
        self._fields_container = Output(
            forward_fields_container._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self) -> Output:
        r"""Allows to get fields_container output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.forward_fields_container()
        >>> # Get the output from op.outputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """
        return self._fields_container
