"""
incremental_concatenate_as_fc

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class incremental_concatenate_as_fc(Operator):
    r"""Assemble fields in a fields container.


    Parameters
    ----------
    field: Field
    label: dict, optional
        Label space value that must be applied to the added field.

    Returns
    -------
    output: FieldsContainer

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.incremental_concatenate_as_fc()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_label = dict()
    >>> op.inputs.label.connect(my_label)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.incremental_concatenate_as_fc(
    ...     field=my_field,
    ...     label=my_label,
    ... )

    >>> # Get output data
    >>> result_output = op.outputs.output()
    """

    def __init__(self, field=None, label=None, config=None, server=None):
        super().__init__(
            name="incremental::concatenate_as_fc", config=config, server=server
        )
        self._inputs = InputsIncrementalConcatenateAsFc(self)
        self._outputs = OutputsIncrementalConcatenateAsFc(self)
        if field is not None:
            self.inputs.field.connect(field)
        if label is not None:
            self.inputs.label.connect(label)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Assemble fields in a fields container.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="label",
                    type_names=["label_space"],
                    optional=True,
                    document=r"""Label space value that must be applied to the added field.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="output",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(
            name="incremental::concatenate_as_fc", server=server
        )

    @property
    def inputs(self) -> InputsIncrementalConcatenateAsFc:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsIncrementalConcatenateAsFc.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsIncrementalConcatenateAsFc:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsIncrementalConcatenateAsFc.
        """
        return super().outputs


class InputsIncrementalConcatenateAsFc(_Inputs):
    """Intermediate class used to connect user inputs to
    incremental_concatenate_as_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.incremental_concatenate_as_fc()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_label = dict()
    >>> op.inputs.label.connect(my_label)
    """

    def __init__(self, op: Operator):
        super().__init__(incremental_concatenate_as_fc._spec().inputs, op)
        self._field = Input(
            incremental_concatenate_as_fc._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._field)
        self._label = Input(
            incremental_concatenate_as_fc._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._label)

    @property
    def field(self) -> Input:
        r"""Allows to connect field input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.incremental_concatenate_as_fc()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def label(self) -> Input:
        r"""Allows to connect label input to the operator.

        Label space value that must be applied to the added field.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.incremental_concatenate_as_fc()
        >>> op.inputs.label.connect(my_label)
        >>> # or
        >>> op.inputs.label(my_label)
        """
        return self._label


class OutputsIncrementalConcatenateAsFc(_Outputs):
    """Intermediate class used to get outputs from
    incremental_concatenate_as_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.incremental_concatenate_as_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_output = op.outputs.output()
    """

    def __init__(self, op: Operator):
        super().__init__(incremental_concatenate_as_fc._spec().outputs, op)
        self._output = Output(
            incremental_concatenate_as_fc._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._output)

    @property
    def output(self) -> Output:
        r"""Allows to get output output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.incremental_concatenate_as_fc()
        >>> # Get the output from op.outputs. ...
        >>> result_output = op.outputs.output()
        """
        return self._output
