"""
make_overall

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class make_overall(Operator):
    r"""Extracts a value from a field and makes a new field containing only this
    value, with the associated scoping’s location set as ‘overall’.


    Parameters
    ----------
    field: Field
    id: int

    Returns
    -------
    field: Field

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.make_overall()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_id = int()
    >>> op.inputs.id.connect(my_id)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.make_overall(
    ...     field=my_field,
    ...     id=my_id,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(self, field=None, id=None, config=None, server=None):
        super().__init__(name="make_overall", config=config, server=server)
        self._inputs = InputsMakeOverall(self)
        self._outputs = OutputsMakeOverall(self)
        if field is not None:
            self.inputs.field.connect(field)
        if id is not None:
            self.inputs.id.connect(id)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Extracts a value from a field and makes a new field containing only this
value, with the associated scoping’s location set as ‘overall’.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="id",
                    type_names=["int32"],
                    optional=False,
                    document=r"""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="make_overall", server=server)

    @property
    def inputs(self) -> InputsMakeOverall:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsMakeOverall.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsMakeOverall:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsMakeOverall.
        """
        return super().outputs


class InputsMakeOverall(_Inputs):
    """Intermediate class used to connect user inputs to
    make_overall operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.make_overall()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_id = int()
    >>> op.inputs.id.connect(my_id)
    """

    def __init__(self, op: Operator):
        super().__init__(make_overall._spec().inputs, op)
        self._field = Input(make_overall._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._id = Input(make_overall._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._id)

    @property
    def field(self) -> Input:
        r"""Allows to connect field input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_overall()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def id(self) -> Input:
        r"""Allows to connect id input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_overall()
        >>> op.inputs.id.connect(my_id)
        >>> # or
        >>> op.inputs.id(my_id)
        """
        return self._id


class OutputsMakeOverall(_Outputs):
    """Intermediate class used to get outputs from
    make_overall operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.make_overall()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(make_overall._spec().outputs, op)
        self._field = Output(make_overall._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self) -> Output:
        r"""Allows to get field output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.make_overall()
        >>> # Get the output from op.outputs. ...
        >>> result_field = op.outputs.field()
        """
        return self._field
