"""
merge_property_fields

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class merge_property_fields(Operator):
    r"""Assembles a set of property fields into a unique one.


    Parameters
    ----------
    naive_merge: bool
        If true, merge the input property fields assuming that there is no repetition in their scoping ids. Default is false.
    property_fields1: PropertyField or PropertyFieldsContainer
        Either a property fields container, a vector of property fields to merge or property fields from pin 0 to ...
    property_fields2: PropertyField or PropertyFieldsContainer
        Either a property fields container, a vector of property fields to merge or property fields from pin 0 to ...

    Returns
    -------
    property_field: PropertyField

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.merge_property_fields()

    >>> # Make input connections
    >>> my_naive_merge = bool()
    >>> op.inputs.naive_merge.connect(my_naive_merge)
    >>> my_property_fields1 = dpf.PropertyField()
    >>> op.inputs.property_fields1.connect(my_property_fields1)
    >>> my_property_fields2 = dpf.PropertyField()
    >>> op.inputs.property_fields2.connect(my_property_fields2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.merge_property_fields(
    ...     naive_merge=my_naive_merge,
    ...     property_fields1=my_property_fields1,
    ...     property_fields2=my_property_fields2,
    ... )

    >>> # Get output data
    >>> result_property_field = op.outputs.property_field()
    """

    def __init__(
        self,
        naive_merge=None,
        property_fields1=None,
        property_fields2=None,
        config=None,
        server=None,
    ):
        super().__init__(name="merge::property_field", config=config, server=server)
        self._inputs = InputsMergePropertyFields(self)
        self._outputs = OutputsMergePropertyFields(self)
        if naive_merge is not None:
            self.inputs.naive_merge.connect(naive_merge)
        if property_fields1 is not None:
            self.inputs.property_fields1.connect(property_fields1)
        if property_fields2 is not None:
            self.inputs.property_fields2.connect(property_fields2)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Assembles a set of property fields into a unique one.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                -201: PinSpecification(
                    name="naive_merge",
                    type_names=["bool"],
                    optional=False,
                    document=r"""If true, merge the input property fields assuming that there is no repetition in their scoping ids. Default is false.""",
                ),
                0: PinSpecification(
                    name="property_fields",
                    type_names=["property_field", "property_fields_container"],
                    optional=False,
                    document=r"""Either a property fields container, a vector of property fields to merge or property fields from pin 0 to ...""",
                ),
                1: PinSpecification(
                    name="property_fields",
                    type_names=["property_field", "property_fields_container"],
                    optional=False,
                    document=r"""Either a property fields container, a vector of property fields to merge or property fields from pin 0 to ...""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="property_field",
                    type_names=["property_field"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="merge::property_field", server=server)

    @property
    def inputs(self) -> InputsMergePropertyFields:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsMergePropertyFields.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsMergePropertyFields:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsMergePropertyFields.
        """
        return super().outputs


class InputsMergePropertyFields(_Inputs):
    """Intermediate class used to connect user inputs to
    merge_property_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_property_fields()
    >>> my_naive_merge = bool()
    >>> op.inputs.naive_merge.connect(my_naive_merge)
    >>> my_property_fields1 = dpf.PropertyField()
    >>> op.inputs.property_fields1.connect(my_property_fields1)
    >>> my_property_fields2 = dpf.PropertyField()
    >>> op.inputs.property_fields2.connect(my_property_fields2)
    """

    def __init__(self, op: Operator):
        super().__init__(merge_property_fields._spec().inputs, op)
        self._naive_merge = Input(
            merge_property_fields._spec().input_pin(-201), -201, op, -1
        )
        self._inputs.append(self._naive_merge)
        self._property_fields1 = Input(
            merge_property_fields._spec().input_pin(0), 0, op, 0
        )
        self._inputs.append(self._property_fields1)
        self._property_fields2 = Input(
            merge_property_fields._spec().input_pin(1), 1, op, 1
        )
        self._inputs.append(self._property_fields2)

    @property
    def naive_merge(self) -> Input:
        r"""Allows to connect naive_merge input to the operator.

        If true, merge the input property fields assuming that there is no repetition in their scoping ids. Default is false.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_property_fields()
        >>> op.inputs.naive_merge.connect(my_naive_merge)
        >>> # or
        >>> op.inputs.naive_merge(my_naive_merge)
        """
        return self._naive_merge

    @property
    def property_fields1(self) -> Input:
        r"""Allows to connect property_fields1 input to the operator.

        Either a property fields container, a vector of property fields to merge or property fields from pin 0 to ...

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_property_fields()
        >>> op.inputs.property_fields1.connect(my_property_fields1)
        >>> # or
        >>> op.inputs.property_fields1(my_property_fields1)
        """
        return self._property_fields1

    @property
    def property_fields2(self) -> Input:
        r"""Allows to connect property_fields2 input to the operator.

        Either a property fields container, a vector of property fields to merge or property fields from pin 0 to ...

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_property_fields()
        >>> op.inputs.property_fields2.connect(my_property_fields2)
        >>> # or
        >>> op.inputs.property_fields2(my_property_fields2)
        """
        return self._property_fields2


class OutputsMergePropertyFields(_Outputs):
    """Intermediate class used to get outputs from
    merge_property_fields operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_property_fields()
    >>> # Connect inputs : op.inputs. ...
    >>> result_property_field = op.outputs.property_field()
    """

    def __init__(self, op: Operator):
        super().__init__(merge_property_fields._spec().outputs, op)
        self._property_field = Output(
            merge_property_fields._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._property_field)

    @property
    def property_field(self) -> Output:
        r"""Allows to get property_field output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_property_fields()
        >>> # Get the output from op.outputs. ...
        >>> result_property_field = op.outputs.property_field()
        """
        return self._property_field
