"""
merge_to_field_matrix

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class merge_to_field_matrix(Operator):
    r"""Assembles a set of fields into a field matrix.


    Parameters
    ----------
    fields1: Field or FieldsContainer
        Either a fields container, a vector of fields to merge, or fields from pin 0 to ...
    fields2: Field or FieldsContainer
        Either a fields container, a vector of fields to merge, or fields from pin 0 to ...

    Returns
    -------
    merged_field_matrix: Field

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.merge_to_field_matrix()

    >>> # Make input connections
    >>> my_fields1 = dpf.Field()
    >>> op.inputs.fields1.connect(my_fields1)
    >>> my_fields2 = dpf.Field()
    >>> op.inputs.fields2.connect(my_fields2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.merge_to_field_matrix(
    ...     fields1=my_fields1,
    ...     fields2=my_fields2,
    ... )

    >>> # Get output data
    >>> result_merged_field_matrix = op.outputs.merged_field_matrix()
    """

    def __init__(self, fields1=None, fields2=None, config=None, server=None):
        super().__init__(name="merge::to_field_matrix", config=config, server=server)
        self._inputs = InputsMergeToFieldMatrix(self)
        self._outputs = OutputsMergeToFieldMatrix(self)
        if fields1 is not None:
            self.inputs.fields1.connect(fields1)
        if fields2 is not None:
            self.inputs.fields2.connect(fields2)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Assembles a set of fields into a field matrix.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document=r"""Either a fields container, a vector of fields to merge, or fields from pin 0 to ...""",
                ),
                1: PinSpecification(
                    name="fields",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document=r"""Either a fields container, a vector of fields to merge, or fields from pin 0 to ...""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="merged_field_matrix",
                    type_names=["field"],
                    optional=False,
                    document=r"""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(name="merge::to_field_matrix", server=server)

    @property
    def inputs(self) -> InputsMergeToFieldMatrix:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsMergeToFieldMatrix.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsMergeToFieldMatrix:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsMergeToFieldMatrix.
        """
        return super().outputs


class InputsMergeToFieldMatrix(_Inputs):
    """Intermediate class used to connect user inputs to
    merge_to_field_matrix operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_to_field_matrix()
    >>> my_fields1 = dpf.Field()
    >>> op.inputs.fields1.connect(my_fields1)
    >>> my_fields2 = dpf.Field()
    >>> op.inputs.fields2.connect(my_fields2)
    """

    def __init__(self, op: Operator):
        super().__init__(merge_to_field_matrix._spec().inputs, op)
        self._fields1 = Input(merge_to_field_matrix._spec().input_pin(0), 0, op, 0)
        self._inputs.append(self._fields1)
        self._fields2 = Input(merge_to_field_matrix._spec().input_pin(1), 1, op, 1)
        self._inputs.append(self._fields2)

    @property
    def fields1(self) -> Input:
        r"""Allows to connect fields1 input to the operator.

        Either a fields container, a vector of fields to merge, or fields from pin 0 to ...

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_to_field_matrix()
        >>> op.inputs.fields1.connect(my_fields1)
        >>> # or
        >>> op.inputs.fields1(my_fields1)
        """
        return self._fields1

    @property
    def fields2(self) -> Input:
        r"""Allows to connect fields2 input to the operator.

        Either a fields container, a vector of fields to merge, or fields from pin 0 to ...

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_to_field_matrix()
        >>> op.inputs.fields2.connect(my_fields2)
        >>> # or
        >>> op.inputs.fields2(my_fields2)
        """
        return self._fields2


class OutputsMergeToFieldMatrix(_Outputs):
    """Intermediate class used to get outputs from
    merge_to_field_matrix operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_to_field_matrix()
    >>> # Connect inputs : op.inputs. ...
    >>> result_merged_field_matrix = op.outputs.merged_field_matrix()
    """

    def __init__(self, op: Operator):
        super().__init__(merge_to_field_matrix._spec().outputs, op)
        self._merged_field_matrix = Output(
            merge_to_field_matrix._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._merged_field_matrix)

    @property
    def merged_field_matrix(self) -> Output:
        r"""Allows to get merged_field_matrix output of the operator

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_to_field_matrix()
        >>> # Get the output from op.outputs. ...
        >>> result_merged_field_matrix = op.outputs.merged_field_matrix()
        """
        return self._merged_field_matrix
