"""
set_attribute

Autogenerated DPF operator classes.
"""

from __future__ import annotations

from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification
from ansys.dpf.core.config import Config
from ansys.dpf.core.server_types import AnyServerType


class set_attribute(Operator):
    r"""Uses the FieldsContainer APIs to modify it.


    Parameters
    ----------
    fields_container: FieldsContainer
    property_name: str
        Supported property names are: "labels".
    property_identifier: dict, optional
        Value of the property to be set : vector of string or LabelSpace for "labels".

    Returns
    -------
    fields_container: FieldsContainer
        Returns the modified FieldsContainer.

    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.set_attribute()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_identifier = dict()
    >>> op.inputs.property_identifier.connect(my_property_identifier)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.set_attribute(
    ...     fields_container=my_fields_container,
    ...     property_name=my_property_name,
    ...     property_identifier=my_property_identifier,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        fields_container=None,
        property_name=None,
        property_identifier=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="fieldscontainer::set_attribute", config=config, server=server
        )
        self._inputs = InputsSetAttribute(self)
        self._outputs = OutputsSetAttribute(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if property_name is not None:
            self.inputs.property_name.connect(property_name)
        if property_identifier is not None:
            self.inputs.property_identifier.connect(property_identifier)

    @staticmethod
    def _spec() -> Specification:
        description = r"""Uses the FieldsContainer APIs to modify it.
"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""""",
                ),
                1: PinSpecification(
                    name="property_name",
                    type_names=["string"],
                    optional=False,
                    document=r"""Supported property names are: "labels".""",
                ),
                2: PinSpecification(
                    name="property_identifier",
                    type_names=["vector<string>", "label_space"],
                    optional=True,
                    document=r"""Value of the property to be set : vector of string or LabelSpace for "labels".""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document=r"""Returns the modified FieldsContainer.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server: AnyServerType = None) -> Config:
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server:
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.

        Returns
        -------
        config:
            A new Config instance equivalent to the default config for this operator.
        """
        return Operator.default_config(
            name="fieldscontainer::set_attribute", server=server
        )

    @property
    def inputs(self) -> InputsSetAttribute:
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs:
            An instance of InputsSetAttribute.
        """
        return super().inputs

    @property
    def outputs(self) -> OutputsSetAttribute:
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs:
            An instance of OutputsSetAttribute.
        """
        return super().outputs


class InputsSetAttribute(_Inputs):
    """Intermediate class used to connect user inputs to
    set_attribute operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.set_attribute()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    >>> my_property_identifier = dict()
    >>> op.inputs.property_identifier.connect(my_property_identifier)
    """

    def __init__(self, op: Operator):
        super().__init__(set_attribute._spec().inputs, op)
        self._fields_container = Input(set_attribute._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._fields_container)
        self._property_name = Input(set_attribute._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._property_name)
        self._property_identifier = Input(set_attribute._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._property_identifier)

    @property
    def fields_container(self) -> Input:
        r"""Allows to connect fields_container input to the operator.

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.set_attribute()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def property_name(self) -> Input:
        r"""Allows to connect property_name input to the operator.

        Supported property names are: "labels".

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.set_attribute()
        >>> op.inputs.property_name.connect(my_property_name)
        >>> # or
        >>> op.inputs.property_name(my_property_name)
        """
        return self._property_name

    @property
    def property_identifier(self) -> Input:
        r"""Allows to connect property_identifier input to the operator.

        Value of the property to be set : vector of string or LabelSpace for "labels".

        Returns
        -------
        input:
            An Input instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.set_attribute()
        >>> op.inputs.property_identifier.connect(my_property_identifier)
        >>> # or
        >>> op.inputs.property_identifier(my_property_identifier)
        """
        return self._property_identifier


class OutputsSetAttribute(_Outputs):
    """Intermediate class used to get outputs from
    set_attribute operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.set_attribute()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(set_attribute._spec().outputs, op)
        self._fields_container = Output(set_attribute._spec().output_pin(0), 0, op)
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self) -> Output:
        r"""Allows to get fields_container output of the operator

        Returns the modified FieldsContainer.

        Returns
        -------
        output:
            An Output instance for this pin.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.set_attribute()
        >>> # Get the output from op.outputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """
        return self._fields_container
